/*
    Nakude dushata mi da byaga
    I kude da skrie svoite sulzi
*/

#include "pointsort.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <random>
#include <bitset>
#include <map>

typedef long long llong;
const int MAXN = 10000 + 10;
const int MAXLOG = 18;
const int INF = 1e9;

int n, k;
struct Info
{
    int x, y;
    int mask;

    inline bool cmp(int x) const
    {
        return (mask & (1 << x));
    }
};

int query(int x, int y)
{
    int mask = 0;
    std::vector <bool> res = compare(x, y);
    for (int i = k - 1 ; i >= 0 ; --i)
    {
        mask <<= 1;
        mask += res[i];
    }

    return mask;
}

int dep[MAXN];
bool hasParent[MAXN];
std::vector <int> g[MAXN];
std::bitset <MAXN> bs[MAXN];
std::vector <int> g2[MAXN];
std::vector <int> toClear;
std::vector <Info> info;
std::mt19937 rng(69420);

int vis[MAXN];
int timer;

void rebuild(int node)
{
    vis[node] = timer;
    bs[node].reset();
    bs[node][node] = 1;
    int idx = 0;
    for (const int &u : g[node])
    {
        if (vis[u] != timer)
        {
            rebuild(u);
            bs[node] |= bs[u];
        }
    }

}

void dfs2(int node)
{
    vis[node] = timer;
    for (const int &u : g2[node])
    {
        if (vis[u] != timer)
        {
            std::bitset <MAXN> old = bs[u];
            bs[u] |= bs[node];
            if (old != bs[u]) dfs2(u);
        }
    }

}

std::vector <std::vector <int>> pointSort(int N, int K)
{
    n = N; k = K;
    info.clear();
    std::vector <std::vector <int>> ans;
    std::vector <std::vector <int>> sol;
    ans.resize(k);

    for (int run = 0 ; run < k ; ++run)
    {
        std::fill(dep, dep + n, 1);
        for (int i = 0 ; i < n ; ++i)
        {
            g[i].clear();
            g2[i].clear();
        }

        ans[run].resize(n);
        std::fill(hasParent, hasParent + n, false);
        for (const auto &curr : info)
        {
            if (curr.cmp(run))
            {
                dep[curr.x] = std::max(dep[curr.x], dep[curr.y] + 1);
                g[curr.x].push_back(curr.y);
                hasParent[curr.y] = true;
            } else
            {
                dep[curr.y] = std::max(dep[curr.y], dep[curr.x] + 1);
                g[curr.y].push_back(curr.x);
                hasParent[curr.x] = true;
            }
        }

        timer++;
        for (int i = 1 ; i <= n ; ++i)
        {
            if (vis[i] != timer)
            {
                rebuild(i);
            }
        }

        for (int i = 1 ; i <= n ; ++i)
        {
            std::bitset <MAXN> curr;
            std::sort(g[i].begin(), g[i].end(), [&](int x, int y)
            {
                return bs[x].count() > bs[y].count(); 
            });
            
            std::vector <int> toRemove;
            for (int j = 0 ; j < g[i].size() ; ++j)
            {
                if (!curr[g[i][j]])
                {
                    curr |= bs[g[i][j]];
                    g2[i].push_back(g[i][j]);
                }
            }

            g[i] = g2[i];
        }

        for (int i = 1 ; i <= n ; ++i)
        {
            g2[i].clear();
        }

        for (int i = 1 ; i <= n ; ++i)
        {
            for (const int &u : g[i])
            {
                g2[u].push_back(i);
            }
        }

        std::vector <int> heads;
        for (int i = 0 ; i < n ; ++i)
        {
            if (!hasParent[i])
            {
                heads.push_back(i);
            }
        }
        
        int ptr = 0;
        int cntEdges = 0;
        while (ptr < n)
        {
            assert(heads.size());
            if (heads.size() == 1)
            {
                ans[run][ptr] = heads[0];
                int root = heads[0];
                heads.pop_back();
                std::sort(g[root].begin(), g[root].end(), [&](int x, int y)
                {
                    return bs[x].count() > bs[y].count(); 
                });

                std::bitset <MAXN> orOfRoots;
                for (const int &u : g[root])
                {
                    if (!orOfRoots[u])
                    {
                        heads.push_back(u);
                        orOfRoots |= bs[u];
                    }
                }

                ptr++;
                continue;
            }
        

            int idxOne = 0;
            int idxTwo = 1;
            if (dep[heads[idxTwo]] < dep[heads[idxOne]])
            {
                std::swap(idxOne, idxTwo);
            }

            for (int i = 2 ; i < heads.size() ; ++i)
            {
                if (dep[heads[i]] < dep[heads[idxOne]])
                {
                    idxTwo = idxOne;
                    idxOne = i;
                } else if (dep[heads[i]] < dep[heads[idxTwo]])
                {
                    idxTwo = i;
                }
            }

            timer++;
            bool firstRes = bs[heads[idxOne]][heads[idxTwo]];
            timer++;
            bool secondRes = bs[heads[idxTwo]][heads[idxOne]];
            bool isDone = (firstRes || secondRes);
            bool res = firstRes;
            if (!isDone)
            {
                int curr = query(heads[idxOne], heads[idxTwo]);
                info.push_back({heads[idxOne], heads[idxTwo], curr});
                res = info.back().cmp(run);
            }

            if (res)
            {
                dep[heads[idxOne]] = std::max(dep[heads[idxOne]], 1 + dep[heads[idxTwo]]);
                if (!isDone) 
                {
                    g[heads[idxOne]].push_back(heads[idxTwo]);
                    g2[heads[idxTwo]].push_back(heads[idxOne]);
                    bs[heads[idxOne]] |= bs[heads[idxTwo]]; timer++;
                    dfs2(heads[idxOne]);
                }
                
                std::swap(heads.back(), heads[idxTwo]);
                heads.pop_back();
            } else
            {
                dep[heads[idxTwo]] = std::max(1 + dep[heads[idxOne]], dep[heads[idxTwo]]);
                if (!isDone) 
                {
                    g[heads[idxTwo]].push_back(heads[idxOne]);
                    g2[heads[idxOne]].push_back(heads[idxTwo]);
                    bs[heads[idxTwo]] |= bs[heads[idxOne]]; timer++;
                    dfs2(heads[idxTwo]);
                }
                std::swap(heads.back(), heads[idxOne]);
                heads.pop_back();
            }
        }
    }

    sol.resize(n);
    for (int i = 0 ; i < n ; ++i)
    {
        sol[i].resize(k);
    }

    for (int i = 0 ; i < k ; ++i)
    {
        for (int j = 0 ; j < n ; ++j)
        {
            sol[ans[i][j]][i] = j;
        }
    }

    return sol;
}