/*
    Nakude dushata mi da byaga
    I kude da skrie svoite sulzi
*/

#include "candies.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>

typedef long long llong;
const int MAXN = 200000 + 10;
const int MAXLOG = 18;
const int INF = 1e9;

int n;
struct SegmentTree
{
    struct Node
    {
        llong ansL;
        llong ansR;
        llong ansMe;
        llong ansAdd;
        llong lazy;
        llong b;
        llong p;
        bool active;
    
        Node()
        {
            active = false;
            ansL = ansR = ansMe = ansAdd = INF;
            lazy = 0;
        }
        
        Node(llong p, llong b)
        {
            active = true;
            ansL = ansR = INF;
            ansMe = b - p;
            ansAdd = p;
            lazy = 0;
        }
        
        void set(llong p, llong b)
        {
            active = true;
            ansL = ansR = INF;
            ansMe = b - p;
            ansAdd = p;
            lazy = 0;
        }

        inline friend Node operator + (const Node &left, const Node &right)
        {
            assert(right.active);
            if (!left.active)
            {
                return right;
            }

            if (!right.active)
            {
                return left;
            }

            Node result;
            result.active = true;
            result.ansMe = std::min(left.ansMe, right.ansMe);
            result.ansAdd = std::min(left.ansAdd, right.ansAdd);
            result.ansL = std::min(left.ansL, right.ansL);
            result.ansR = std::min(left.ansR, right.ansR);
            result.ansL = std::min(result.ansL, left.ansMe + right.ansAdd);
            result.ansR = std::min(result.ansR, right.ansMe + left.ansAdd);
            return result;
        }
    };

    Node tree[4 * MAXN];
    void build(int l, int r, int node, llong p[], llong b[])
    {
        if (l == r)
        {
            tree[node].set(p[l], b[l]);
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node, p, b);
        build(mid + 1, r, 2*node + 1, p, b);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    void push(int node, int l, int r)
    {
        if (tree[node].lazy == 0)
        {
            return;
        }

        tree[node].ansMe -= tree[node].lazy;
        tree[node].ansAdd += tree[node].lazy;

        if (l != r)
        {
            tree[2*node].lazy += tree[node].lazy;
            tree[2*node + 1].lazy += tree[node].lazy;
        }

        tree[node].lazy = 0;
    }

    void updateB(int l, int r, int node, int queryPos, int queryVal)
    {
        push(node, l, r);
        if (queryPos < l || r < queryPos)
        {
            return;
        }

        if (l == r)
        {
            tree[node].set(tree[node].p, tree[node].b);
            return;
        }

        int mid = l + r >> 1;
        updateB(l, mid, 2*node, queryPos, queryVal);
        updateB(mid + 1, r, 2*node + 1, queryPos, queryVal);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    void update(int l, int r, int node, int queryL, int queryR, llong queryVal)
    {
        push(node, l, r);
        if (queryR < l || r < queryL)
        {
            return;
        }

        if (queryL <= l && r <= queryR)
        {
            tree[node].lazy = queryVal;
            push(node, l, r);
            return;
        }

        int mid = l + r >> 1;
        update(l, mid, 2*node, queryL, queryR, queryVal);
        update(mid + 1, r, 2*node + 1, queryL, queryR, queryVal);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    Node query(int l, int r, int node, int queryL, int queryR)
    {
        push(node, l, r);
        if (queryL <= l && r <= queryR)
        {
            return tree[node];
        }

        Node res;
        int mid = l + r >> 1;
        if (queryL <= mid) res = res + query(l, mid, 2*node, queryL, queryR);
        if (mid + 1 <= queryR) res = res + query(mid + 1, r, 2*node + 1, queryL, queryR);
        return res;
    }

    llong getP(int l, int r, int node, int queryPos)
    {
        push(node, l, r);
        if (l == r)
        {
            return tree[node].p;
        }

        int mid = l + r >> 1;
        if (queryPos <= mid) return getP(l, mid, 2*node, queryPos);
        else return getP(mid + 1, r, 2*node + 1, queryPos);
    }

    void build(llong p[], llong b[])
    {
        build(1, n, 1, p, b);
    }

    Node query(int l, int r)
    {
        return query(1, n, 1, l, r);
    }
};

// int s[MAXN];
// int c[MAXN];
llong p[MAXN];
llong b[MAXN];
llong a[MAXN];
SegmentTree tree;
void init(std::vector <int> A, std::vector <int> B)
{
    n = A.size();
    for (int i = 1 ; i <= n ; ++i)
    {
        a[i] = A[i - 1];
        b[i] = B[i - 1];
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        p[i] = b[i] - a[i] + p[i - 1];
    }

    tree.build(p, b);
}

bool isValid(int l, int r)
{
    l++; r++;
    // std::cout << "here2: " << p[r] - p[l - 1] << '\n'; 
    if (tree.getP(1, n, 1, r) - (l == 1 ? 0 : tree.getP(1, n, 1, l - 1)) < 0)
    {
        return 0;
    }

    SegmentTree::Node res = tree.query(l, r);
    // std::cout << "here: " << res.ansL << ' ' << res.ansR << '\n'; 
    if (res.ansL < 0 || res.ansR + p[r] - p[l - 1] < 0)
    {
        return false;
    }

    return true;
}

void update(int l, int r, int x, int y)
{
    l++; r++;
    assert(l == r);
    llong old = tree.getP(1, n, 1, l);
    if (l > 1) old -= tree.getP(1, n, 1, l - 1);
    tree.updateB(1, n, 1, l, y);
    tree.update(1, n, 1, l, n, (y - x) - old);
}