#include <iostream>
#include <vector>
#include "candies.h"

const int MAXN = 2 * 1e5 + 1;
const long long MAXA = 1e9 + 1;
long long arr[2][MAXN];
int n;

struct Ver{
    long long val;
    long long lazy;
};
Ver tree[2][8 * MAXN];

void build(int curr, int from, int to, int flip) {
    if (from == to) {
        tree[flip][curr].val = arr[flip][from];
        tree[flip][curr].lazy = 0;
    } else {
        build(curr * 2, from, (from + to) / 2, flip);
        build(curr * 2 + 1, (from + to) / 2 + 1, to, flip);
        tree[flip][curr].val = tree[flip][curr * 2].val + tree[flip][curr * 2 + 1].val;
        tree[flip][curr].lazy = 0;
    }
}
inline void push(int curr, int from, int to) {
    for (int i = 0; i <= 1; i++) {
        if (tree[i][curr].lazy != 0) {
            tree[i][curr].val = tree[i][curr].lazy * (to - from + 1);
            tree[i][curr * 2].lazy = tree[i][curr].lazy;
            tree[i][curr * 2 + 1].lazy = tree[i][curr].lazy;
            tree[i][curr].lazy = 0;
        }
    }
}
void seg_update(int curr, int from, int to, int l, int r, long long x, long long y) {
    if (tree[0][curr].lazy != 0 || tree[1][curr].lazy != 0) {
        push(curr, from, to);
    }
    if (l <= from && to <= r) {
        tree[0][curr].lazy = x;
        tree[1][curr].lazy = y;
        push(curr, from, to);
    } else if (l <= to && from <= r) {
        seg_update(curr * 2, from, (from + to) / 2, l, r, x, y);
        seg_update(curr * 2 + 1, (from + to) / 2 + 1, to, l, r, x, y);
        for (int i = 0; i <= 1; i++) {
            tree[i][curr].val = tree[i][curr * 2].val + tree[i][curr * 2 + 1].val;
        }
    }
}

long long query(int curr, int from, int to, int l, int r, int flip) {
    if (tree[flip][curr].lazy != 0) {
        push(curr, from, to);
    }
    if (l <= from && to <= r) {
        return tree[flip][curr].val;
    } else if (l <= to && from <= r) {
        return query(curr * 2, from, (from + to) / 2, l, r, flip) + query(curr * 2 + 1, (from + to) / 2 + 1, to, l, r, flip);
    } else {
        return 0;
    }
}

void init(std::vector<int> kids, const std::vector<int> candies) {
    n = kids.size();
    for (int i = 1; i <= n; i++) {
        arr[0][i] = kids[i - 1];
        arr[1][i] = candies[i - 1];
    }
    build(1, 1, n, 0);
    build(1, 1, n, 1);
}

bool isValid(int l, int r) {
    l++;
    r++;
    if (query(1, 1, n, l, r, 0) > query(1, 1, n, l, r, 1)) {
        return false;
    }
    if (query(1, 1, n, l, l, 0) > query(1, 1, n, l, l, 1) + query(1, 1, n, r, r, 1)) {
        return false;
    }
    for (int i = l + 1; i <= r; i++) {
        if (query(1, 1, n, i, i, 0) > query(1, 1, n, i - 1, i, 1)) {
            return false;
        }
    }
    for (int i = l; i <= r; i++) {
        for (int j = i + 1; j <= r; j++) {
            if (j - i == r - l) {
                continue;
            }
            if (query(1, 1, n, i, j, 0) > query(1, 1, n, i - 1 + (i == l), j, 1) + (i == l) * query(1, 1, n, r, r, 1)) {
                return false;
            }
            if (i == l || j == r) {
                continue;
            }
            if (query(1, 1, n, l, i - 1, 0) + query(1, 1, n, j + 1, r, 0) > query(1, 1, n, l, i - 1, 1) + query(1, 1, n, j, r, 1)) {
                return false;
            }
        }
    }
    return true;
}

void update(int l, int r, int x, int y) {
    seg_update(1, 1, n, l + 1, r + 1, x, y);
}

//int main() {
//    std::ios_base::sync_with_stdio(false);
//    std::cin.tie(0);
//    std::cout.tie(0);
//    int nn;
//    std::cin >> nn;
//    std::vector<int> children, machines;
//    for (int i = 1; i <= nn; i++) {
//        int v;
//        std::cin >> v;
//        children.push_back(v);
//    }
//    for (int i = 1; i <= nn; i++) {
//        int v;
//        std::cin >> v;
//        machines.push_back(v);
//    }
//    init(children, machines);
//    int q;
//    std::cin >> q;
//    while (q --> 0) {
//        int type;
//        std::cin >> type;
//        if (type == 1) {
//            int l, r;
//            std::cin >> l >> r;
//            std::cout << isValid(l, r) << '\n';
//        } else {
//            int l, r, x, y;
//            std::cin >> l >> r >> x >> y;
//            update(l, r, x, y);
////            for (int j = 0; j <= 1; j++) {
////                for (int i = 1; i <= n; i++) {
////                    std::cout << query(1, 1, n, i, i, j) << ' ';
////                }
////                std::cout << '\n';
////            }
//        }
//    }
//    return 0;
//}
