#include <bits/stdc++.h>
#include "candies.h"

#ifdef __LOCAL__
    #include "Lgrader.cpp"
#endif // __LOCAL__

#define endl '\n'

using namespace std;

const int MAXN = 1e5+50;
class SegmentTree
{

private:
    int N;

    int64_t normal_tree[4*MAXN];
    int64_t shifted_tree[4*MAXN];
    int64_t tree[4*MAXN];

    void Recalculate(int node)
    {
        normal_tree[node] = normal_tree[node*2] + normal_tree[node*2+1];
        shifted_tree[node] = shifted_tree[node*2] + shifted_tree[node*2+1];
        tree[node] = normal_tree[node] + shifted_tree[node];
    }


    void BuildInternal(const vector <int>& arr, int node, int L, int R)
    {
        if (L == R)
        {
            normal_tree[node] = arr[L];
            shifted_tree[node] = arr[(L-1+N)%N];
            tree[node] = normal_tree[node] + shifted_tree[node];
            return;
        }

        BuildInternal(arr, node*2, L, (L+R)/2);
        BuildInternal(arr, node*2+1, (L+R)/2+1, R);

        Recalculate(node);
    }

    void UpdateInternal(const int qPos, const int value, int64_t *arr, int node, int L, int R)
    {
        if (L > qPos || R < qPos)
            return;

        if (L == R)
        {
            arr[node] = value;
            return;
        }

        UpdateInternal(qPos, value, arr, node*2, L, (L+R)/2);
        UpdateInternal(qPos, value, arr, node*2+1, (L+R)/2+1, R);

        Recalculate(node);
    }

    int64_t QueryInternal(const int queryL, const int queryR, int64_t *arr, int node, int L, int R)
    {
        if (L > queryR || R < queryL)
            return 0;

        if (L >= queryL && R <= queryR)
            return arr[node];

        return QueryInternal(queryL, queryR, arr, node*2, L, (L+R)/2) +
                QueryInternal(queryL, queryR, arr, node*2+1, (L+R)/2+1, R);
    }

public:

    void Build(const int N, const vector <int>& arr)
    {
        this->N = N;
        BuildInternal(arr, 1, 0, N-1);
    }

    void Update(int queryLeft, int queryRight, int value)
    {
        for (int pos = queryLeft; pos <= queryRight; pos++)
        {
            UpdateInternal(pos, value, normal_tree, 1, 0, N-1);
            UpdateInternal((pos+1)%N, value, shifted_tree, 1, 0, N-1);
        }
    }

    int64_t Query(int queryLeft, int queryRight)
    {
        return QueryInternal(queryLeft, queryRight, tree, 1, 0, N-1)
                - QueryInternal(queryLeft, queryLeft, shifted_tree, 1, 0, N-1)
                + QueryInternal(queryRight, queryRight, normal_tree, 1, 0, N-1);
    }
};

SegmentTree childrenTree, machinesTree;
void init(std::vector<int> kids, const std::vector<int> candies)
{
    childrenTree.Build(kids.size(), kids);
    machinesTree.Build(candies.size(), candies);
}
bool isValid(int l, int r)
{
    int64_t wanted = childrenTree.Query(l, r);
    int64_t offered = machinesTree.Query(l, r);

    //cout << wanted << " " << offered << endl;
    return offered >= wanted;
}
void update(int l, int r, int x, int y)
{
    childrenTree.Update(l, r, x);
    machinesTree.Update(l, r, y);
}

/*
5
2 0 9 4 1
0 5 4 6 7
7
2 4 4 8 1
1 0 1
2 3 3 4 9
2 1 2 2 2
1 3 4
1 2 2
1 2 4
*/
