#include "pointsort.h"
#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include <random>
#include <math.h>

using namespace std;

const int maxn = 10005;
int best[maxn][5];
int coords[maxn][5];

map<pair<int, int>, vector<bool>> results;

vector<bool> myCompare(int x, int y)
{
    //cout << "here " << x << " " << y << endl;
    if (results.find({x, y}) == results.end() && results.find({y, x}) != results.end())
    {
        vector<bool> curr = results[{y, x}];
        for (int i = 0; i < curr.size(); ++i) curr[i] = 1 - curr[i];
        return results[{x, y}] = curr;
    }
    if (results.find({x, y}) != results.end()) return results[{x, y}];
    return results[{x, y}] = compare(x, y);
}

void mergeSort(int l, int r, int dim)
{
    if (l == r) return;

    int mid = (l + r) / 2;
    mergeSort(l, mid, dim);
    mergeSort(mid + 1, r, dim);

    vector<int> sortedOrder;
    int lPtr = l, rPtr = mid + 1;

    while (lPtr != mid + 1 || rPtr != r + 1)
    {
        if (lPtr == mid + 1)
        {
            sortedOrder.push_back(best[rPtr++][dim]);
        }
        else if (rPtr == r + 1)
        {
            sortedOrder.push_back(best[lPtr++][dim]);
        }
        else
        {
            vector<bool> result = myCompare(best[lPtr][dim], best[rPtr][dim]);
            if (result[dim] == true) sortedOrder.push_back(best[lPtr++][dim]);
            else sortedOrder.push_back(best[rPtr++][dim]);
        }
    }

    for (int i = l; i <= r; ++i)
    {
        best[i][dim] = sortedOrder[i - l];
    }
}

mt19937 mt(69420);

void quickSort(int l, int r, int dim)
{
    if (l > r) return;
    if (l == r) return;

    int pivot = l + mt() % (r - l);
    int pivotReal = best[pivot][dim];
    vector<int> smaller, bigger;

    //cout << l << " " << r << " :: " << pivot << " " << pivotReal << endl;

    for (int i = l; i <= r; ++i)
    {
        if (i == pivot) continue;
        vector<bool> curr = myCompare(best[i][dim], pivotReal);
        if (curr[dim] == 1)
        {
            //cout << i << " " << best[i][dim] << " is smaller " << endl;
            smaller.push_back(best[i][dim]);
        }
        else
        {
            //cout << i << " " << best[i][dim] << " is bigger " << endl;
            bigger.push_back(best[i][dim]);
        }
    }

    int ptr = l;

    for (int i = 0; i < smaller.size(); ++i)
    {
        best[ptr++][dim] = smaller[i];
    }

    best[ptr++][dim] = pivotReal;

    for (int i = 0; i < bigger.size(); ++i)
    {
        best[ptr++][dim] = bigger[i];
    }

    quickSort(l, l + smaller.size() - 1, dim);
    quickSort(r - bigger.size() + 1, r, dim);
}

vector<vector<int>> pointSort(int n, int k)
{
    results.clear();
    for (int i = 0; i < n; ++i)
    {
        for (int j = 0; j < k; ++j)
        {
            best[i][j] = i;
        }
    }

    for (int i = 0; i < k; ++i)
    {
        //cout << "NEW DIMENSION" << endl;
        quickSort(0, n - 1, i);

        for (int j = 0; j < n; ++j)
        {
            coords[best[j][i]][i] = j;
        }
    }

    vector<vector<int>> ans;

    for (int i = 0; i < n; ++i)
    {
        ans.push_back({});

        for (int j = 0; j < k; ++j)
        {
            ans.back().push_back(coords[i][j]);
            //cout << ans[i][j] << "";
        }

        //cout << endl;
    }

    return ans;
}
